/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.menu.navigation;

import java.awt.event.ActionEvent;
import java.util.Observable;

import javax.swing.ImageIcon;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.menu.AbstractModelAction;
import net.sf.jannot.Location;

/**
 * Moves the view of the annotation panel to the left.
 * 
 * @author Thomas Abeel
 * 
 */
public class AnnotationMoveLeftAction extends AbstractModelAction {

    @Override
    public void update(Observable o, Object obj) {
        setEnabled(model.vlm.getAnnotationLocationVisible().start() > 1);
    }

    /**
     * 
     */
    private static final long serialVersionUID = -151723517814284940L;

    public AnnotationMoveLeftAction(Model model) {
        super(null, new ImageIcon(model.getClass().getResource("/images/arrow_left.png")), model);
        
    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
        Location r = model.vlm.getAnnotationLocationVisible();
        int halve = r.length() / 4;
        if(enabled)
        	model.vlm.setAnnotationLocationVisible(new Location(r.start() - halve, r.end() - halve));

    }
    
    public static void perform(Model model){
    	AnnotationMoveLeftAction action = new AnnotationMoveLeftAction(model);
    	action.actionPerformed(null);
    }

}
