/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.viztracks.hts;

import java.util.BitSet;
/**
 * 
 * @author Thomas Abeel
 *
 */
class TilingMatrix {
		/* One place Bitset per pixel column on screen */
		private BitSet[] tc;
		private int visibleLength;

		/* Get a free line for the tranlated genomic coordinate */
		int getFreeLine(int startX) {
			int transX = translate(startX);
			if (transX >= 0 && transX < tc.length)
				return tc[transX].nextClearBit(0);
			else
				return tc[0].nextClearBit(0);

		}

		private int translate(int startX) {
			return (int) (startX * ((double) tc.length / (double) visibleLength));

		}

		public TilingMatrix(double screenWidth, int visibleLength, int maxStack) {
			this.visibleLength = visibleLength;
			tc = new BitSet[(int) screenWidth + 1];
			for (int i = 0; i < tc.length; i++) {
				tc[i] = new BitSet(maxStack);
			}
		}

		public int length() {
			return tc.length;
		}

		/*
		 * set from all x [from,to[
		 */
		public void rangeSet(int fromX, int toX, int y) {
			fromX = translate(fromX);
			toX = translate(toX);
			if (fromX < 0)
				fromX = 0;
			if (toX > tc.length)
				toX = tc.length;
			for (int i = fromX; i < toX; i++) {
				if (i >= 0 && i < tc.length)
					tc[i].set(y);

			}

		}

	}